==> renameutils_0.4.0-2.tar.gz: renameutils-0.4.0/src/editformats/single-column.c <==
/* single-column.c - "single-column" format.
 *
 * Copyright (C) 2001  Oskar Liljeblad
 *
 * This file is part of the file renaming utilities (renameutils).
 *
 * This software is copyrighted work licensed under the terms of the
 * GNU General Public License. Please consult the file `COPYING' for
 * details.
 */

#include <config.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include "common/common.h"
#include "common/io-utils.h"
#include "common/string-utils.h"
#include "common/error.h"
#include "common/memory.h"
#include "common/llist.h"
#include "qmv.h"

static bool parse_options(char *options);
static char *option_generator(const char *text, int state);
static void output(FILE *out, LList *files);
static bool input(FILE *in, LList *files);

static bool swap = FALSE;
static bool separate = FALSE;
static char *indicator1 = "";
static char *indicator2 = "";

EditFormat single_column_format = {
	"single-column",
	"sc",
	parse_options,
	option_generator,
	output,
	input,
};

static char *option_array[] = {
	"help",
	"swap",
	"separate",
	"indicator1",
	"indicator2",
	NULL
};
enum {
	HELP_OPT,
	SWAP_OPT,
	SEPARATE_OPT,
	INDICATOR1_OPT,
	INDICATOR2_OPT
};

#define NO_ARGUMENT(name) \
	if (value != NULL) { \
		warn("suboption `%s' for --option does not take an argument", #name); \
		reset_options(); \
		return FALSE; \
	}
#define REQUIRE_ARGUMENT() \
	if (value == NULL) { \
		warn("missing argument for --option suboption `%s'", option_array[c]); \
		reset_options(); \
		return FALSE; \
	}

static void
reset_options()
{
	swap = FALSE;
	separate = FALSE;
	indicator1 = "";
	indicator2 = "";
}

static bool
parse_options(char *options)
{
	reset_options();

	while (*options != '\0') {
		char *value;
		int c = getsubopt(&options, option_array, &value);

		if (c == -1) {
			warn("unknown suboption for --option: %s", value);
			reset_options();
			return FALSE;
		}

		switch (c) {
		case HELP_OPT:
			printf("\
Available options for `single-column' format:\n\
\n\
  swap              swap location of old and new names when editing\n\
  separate          put a blank line between all renames\n\
  indicator1=TEXT   text to put before the first file name\n\
  indicator2=TEXT   text to put before the second file name\n");
			return FALSE;
		case SWAP_OPT:
			NO_ARGUMENT(swap);
			swap = TRUE;
			break;
		case SEPARATE_OPT:
			NO_ARGUMENT(separate);
			separate = TRUE;
			break;
		case INDICATOR1_OPT:
			REQUIRE_ARGUMENT();
			indicator1 = value;
			break;
		case INDICATOR2_OPT:
			REQUIRE_ARGUMENT();
			indicator2 = value;
			break;
		}
	}

	return TRUE;
}

static char *
option_generator(const char *text, int state)
{
	static int c;
	if (state == 0)
		c = 0;
	while (option_array[c] != NULL) {
		char *name = option_array[c];
		c++;
		if (starts_with(name, text))
			return xstrdup(name);
	}
	return NULL;
}

static void
output(FILE *out, LList *files)
{
	Iterator *it;
	int c = 0;

	for (it = llist_iterator(files); iterator_has_next(it); c++) {
		FileSpec *spec = iterator_next(it);
		char *file1;
		char *file2;

		file1 = quote_output_file(spec->old_name);
		file2 = quote_output_file(spec->new_name);
		fprintf(out, "%s%s\n", indicator1, file1);
		fprintf(out, "%s%s\n", indicator2, file2);
		if (separate && iterator_has_next(it))
			fprintf(out, "\n");
		free(file2);
		free(file1);
	}
	iterator_free(it);
}

static bool
input(FILE *in, LList *files)
{
	Iterator *it;
	int c = 0;

	for (it = llist_iterator(files); iterator_has_next(it); c++) {
		FileSpec *spec = iterator_next(it);
		char *file_old;
		char *file_new;
		char *tmp;

		file_old = read_line(in);
		file_new = read_line(in);
		if (file_old == NULL || file_new == NULL) {
			if (ferror(in))
				warn_errno(NULL);
			else
				warn("premature end of %s", edit_filename);
			iterator_free(it);
			return FALSE;
		}

		if (separate && iterator_has_next(it)) {
			char *empty_line;

			empty_line = read_line(in);
			if (empty_line == NULL) {
				if (ferror(in))
					warn_errno(NULL);
				else
					warn("premature end of %s", edit_filename);
				iterator_free(it);
				return FALSE;
			}
			chomp(empty_line);
			if (empty_line[0] != '\0') {
				warn("expected empty line in file, got `%s'\n", empty_line);
				iterator_free(it);
				return FALSE;
			}
			free(empty_line);
		}

		chomp(file_old);
		chomp(file_new);

		if (!starts_with(file_old, indicator1)) {
			warn("first name for %s did not start with an indicator", spec->old_name);
			iterator_free(it);
			return FALSE;
		}
		tmp = dequote_output_file(file_old + strlen(indicator1));
		free(file_old);
		file_old = tmp;
		if (!starts_with(file_new, indicator2)) {
			warn("second name for %s did not start with an indicator", spec->old_name);
			iterator_free(it);
			return FALSE;
		}
		tmp = dequote_output_file(file_new + strlen(indicator2));
		free(file_new);
		file_new = tmp;

		if (swap) {
			tmp = file_old;
			file_old = file_new;
			file_new = tmp;
		}

		if (strcmp(file_old, spec->old_name) != 0) {
			warn("old filename for %s has been changed", spec->old_name);
			/* Don't return */
		} else if (file_new[0] == '\0') {
			warn("new filename is empty for %s", spec->old_name);
			/* Don't return */
		} else {
			spec->new_name = file_new;
		}
	}
	iterator_free(it);

	return TRUE;
}

==> renameutils_0.5.1.orig.tar.gz: renameutils-0.5.1/src/editformats/single-column.c <==
/* single-column.c - "single-column" format.
 *
 * Copyright (C) 2001  Oskar Liljeblad
 *
 * This file is part of the file renaming utilities (renameutils).
 *
 * This software is copyrighted work licensed under the terms of the
 * GNU General Public License. Please consult the file `COPYING' for
 * details.
 */

#if HAVE_CONFIG_H
#include <config.h>
#endif
/* POSIX/gnulib */
#include <stdbool.h>
/* C89 */
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
/* gettext */
#include <gettext.h> /* will include <libintl.h> if ENABLE_NLS */
#define _(String) gettext(String)
/* common */
#include "common/common.h"
#include "common/io-utils.h"
#include "common/string-utils.h"
#include "common/error.h"
#include "common/memory.h"
#include "common/llist.h"
/* qmv */
#include "qmv.h"

static bool parse_options(char *options);
static char *option_generator(const char *text, int state);
static void output(FILE *out, LList *files);
static bool input(FILE *in, LList *files);

static bool swap = false;
static bool separate = false;
static char *indicator1 = "";
static char *indicator2 = "";

EditFormat single_column_format = {
	"single-column",
	"sc",
	parse_options,
	option_generator,
	output,
	input,
};

static char *option_array[] = {
	"help",
	"swap",
	"separate",
	"indicator1",
	"indicator2",
	NULL
};
enum {
	HELP_OPT,
	SWAP_OPT,
	SEPARATE_OPT,
	INDICATOR1_OPT,
	INDICATOR2_OPT
};

#define NO_ARGUMENT(name) \
	if (value != NULL) { \
		warn(_("suboption `%s' for --option does not take an argument"), #name); \
		reset_options(); \
		return false; \
	}
#define REQUIRE_ARGUMENT() \
	if (value == NULL) { \
		warn(_("missing argument for --option suboption `%s'"), option_array[c]); \
		reset_options(); \
		return false; \
	}

static void
reset_options()
{
	swap = false;
	separate = false;
	indicator1 = "";
	indicator2 = "";
}

static bool
parse_options(char *options)
{
	reset_options();

	while (*options != '\0') {
		char *value;
		int c = getsubopt(&options, option_array, &value);

		if (c == -1) {
			warn(_("unknown suboption for --option: %s"), value);
			reset_options();
			return false;
		}

		switch (c) {
		case HELP_OPT:
			printf(_("\
Available options for `single-column' format:\n\
\n\
  swap              swap location of old and new names when editing\n\
  separate          put a blank line between all renames\n\
  indicator1=TEXT   text to put before the first file name\n\
  indicator2=TEXT   text to put before the second file name\n"));
			return false;
		case SWAP_OPT:
			NO_ARGUMENT(swap);
			swap = true;
			break;
		case SEPARATE_OPT:
			NO_ARGUMENT(separate);
			separate = true;
			break;
		case INDICATOR1_OPT:
			REQUIRE_ARGUMENT();
			indicator1 = value;
			break;
		case INDICATOR2_OPT:
			REQUIRE_ARGUMENT();
			indicator2 = value;
			break;
		}
	}

	return true;
}

static char *
option_generator(const char *text, int state)
{
	static int c;
	if (state == 0)
		c = 0;
	while (option_array[c] != NULL) {
		char *name = option_array[c];
		c++;
		if (starts_with(name, text))
			return xstrdup(name);
	}
	return NULL;
}

static void
output(FILE *out, LList *files)
{
	Iterator *it;
	int c = 0;

	for (it = llist_iterator(files); iterator_has_next(it); c++) {
		FileSpec *spec = iterator_next(it);
		char *file1;
		char *file2;

		file1 = quote_output_file(spec->old_name);
		file2 = quote_output_file(spec->new_name);
		fprintf(out, "%s%s\n", indicator1, file1);
		fprintf(out, "%s%s\n", indicator2, file2);
		if (separate && iterator_has_next(it))
			fprintf(out, "\n");
		free(file2);
		free(file1);
	}
	iterator_free(it);
}

static bool
input(FILE *in, LList *files)
{
	Iterator *it;
	int c = 0;

	for (it = llist_iterator(files); iterator_has_next(it); c++) {
		FileSpec *spec = iterator_next(it);
		char *file_old;
		char *file_new;
		char *tmp;

		file_old = read_line(in);
		file_new = read_line(in);
		if (file_old == NULL || file_new == NULL) {
			if (ferror(in))
				warn_errno(NULL);
			else
				warn(_("premature end of %s"), edit_filename);
			iterator_free(it);
			return false;
		}

		if (separate && iterator_has_next(it)) {
			char *empty_line;

			empty_line = read_line(in);
			if (empty_line == NULL) {
				if (ferror(in))
					warn_errno(NULL);
				else
					warn(_("premature end of %s"), edit_filename);
				iterator_free(it);
				return false;
			}
			chomp(empty_line);
			if (empty_line[0] != '\0') {
				warn(_("expected empty line in file, got `%s'\n"), empty_line);
				iterator_free(it);
				return false;
			}
			free(empty_line);
		}

		chomp(file_old);
		chomp(file_new);

		if (!starts_with(file_old, indicator1)) {
			warn(_("first name for %s did not start with the indicator"), spec->old_name);
			iterator_free(it);
			return false;
		}
		tmp = dequote_output_file(file_old + strlen(indicator1));
		free(file_old);
		file_old = tmp;
		if (!starts_with(file_new, indicator2)) {
			warn(_("second name for %s did not start with the indicator"), spec->old_name);
			iterator_free(it);
			return false;
		}
		tmp = dequote_output_file(file_new + strlen(indicator2));
		free(file_new);
		file_new = tmp;

		if (swap) {
			tmp = file_old;
			file_old = file_new;
			file_new = tmp;
		}

		if (strcmp(file_old, spec->old_name) != 0) {
			warn(_("old filename for %s has been changed"), spec->old_name);
			/* Don't return */
		} else if (file_new[0] == '\0') {
			warn(_("new filename is empty for %s"), spec->old_name);
			/* Don't return */
		} else {
			spec->new_name = file_new;
		}
	}
	iterator_free(it);

	return true;
}
